@@ -21,6 +21,7 @@
 #  include <config.h>
 #endif
 #include "pyorbit-private.h"
+#include <string.h>
 #include <sysmodule.h>
 #include <orbit/poa/poa-types.h>
 #ifdef HAVE_ORBIT2_IMODULE
@@ -121,8 +122,9 @@ static PyMethodDef orbit_functions[] = {
 };
 
 static PyObject *
-pyorbit_corba_orb_init(PyObject *self, PyObject *args)
+pyorbit_corba_orb_init(PyObject *self, PyObject *args, PyObject *kwargs)
 {
+    static char *kwlist[] = { "argv", "orb_id", NULL};
     PyObject *py_argv = NULL;
     gchar *orb_id = "orbit-local-orb";
     int argc, i;
@@ -131,9 +133,10 @@ pyorbit_corba_orb_init(PyObject *self, PyObject *args)
     CORBA_ORB orb;
     PyObject *pyorb;
 
-    if (!PyArg_ParseTuple(args, "|O!s:CORBA.ORB_init",
-			  &PyList_Type, &py_argv, &orb_id))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|O!s:CORBA.ORB_init", kwlist,
+                                     &PyList_Type, &py_argv, &orb_id))
 	return NULL;
+
     if (py_argv && PyList_Size(py_argv) > 0) {
 	argc = PyList_Size(py_argv);
 	argv = g_new(gchar *, argc);
@@ -146,6 +149,7 @@ pyorbit_corba_orb_init(PyObject *self, PyObject *args)
 		g_free(argv);
 		return NULL;
 	    }
+            argv[i] = PyString_AsString(item);
 	}
     } else {
 	argc = 1;
@@ -155,6 +159,11 @@ pyorbit_corba_orb_init(PyObject *self, PyObject *args)
     CORBA_exception_init(&ev);
     orb = CORBA_ORB_init(&argc, argv, orb_id, &ev);
     g_free(argv);
+
+      /* Check if we have a threaded ORB */
+    if (strstr(orb_id, "orbit-io-thread"))
+        PyEval_InitThreads();
+
     _pyorbit_poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
     if (pyorbit_check_ex(&ev))
 	return NULL;
@@ -168,7 +177,7 @@ pyorbit_corba_orb_init(PyObject *self, PyObject *args)
 }
 
 static PyMethodDef corba_functions[] = {
-    { "ORB_init", pyorbit_corba_orb_init, METH_VARARGS },
+    { "ORB_init", (PyCFunction) pyorbit_corba_orb_init, METH_KEYWORDS },
     { NULL, NULL, 0 }
 };
 
@@ -374,11 +383,32 @@ static struct _PyORBit_APIStruct api = {
 
     pyorbit_check_ex,
     pyorbit_marshal_any,
-    pyorbit_demarshal_any
+    pyorbit_demarshal_any,
+    pyorbit_check_python_ex
 };
 
 void initORBit(void);
 
+
+static void
+add_orbit_constants(PyObject *orbitmod)
+{
+#define add_constant(arg) \
+    PyModule_AddIntConstant(orbitmod, #arg, ORBIT_##arg);
+    
+    add_constant(THREAD_HINT_PER_OBJECT);
+    add_constant(THREAD_HINT_PER_POA);
+    add_constant(THREAD_HINT_PER_CONNECTION);
+    add_constant(THREAD_HINT_PER_REQUEST);
+    add_constant(THREAD_HINT_ONEWAY_AT_IDLE);
+    add_constant(THREAD_HINT_ALL_AT_IDLE);
+    add_constant(THREAD_HINT_ON_CONTEXT);
+    add_constant(THREAD_HINT_NONE);
+
+#undef add_constant
+}
+
+
 DL_EXPORT(void)
 initORBit(void)
 {
@@ -404,8 +434,11 @@ initORBit(void)
     INIT_TYPE(PyCORBA_Enum_Type);
     INIT_TYPE(PyCORBA_fixed_Type);
     INIT_TYPE(PyPortableServer_Servant_Type);
+    INIT_TYPE(PyORBit_ObjectAdaptor_Type);
+    PyPortableServer_POA_Type.tp_base = &PyORBit_ObjectAdaptor_Type;
     INIT_TYPE(PyPortableServer_POA_Type);
     INIT_TYPE(PyPortableServer_POAManager_Type);
+    INIT_TYPE(PyCORBA_Policy_Type);
 
 #undef INIT_TYPE
 
@@ -424,9 +457,9 @@ initORBit(void)
 				     PYORBIT_MAJOR_VERSION,
 				     PYORBIT_MINOR_VERSION,
 				     PYORBIT_MICRO_VERSION));
-
     PyModule_AddObject(mod, "_PyORBit_API",
 		       PyCObject_FromVoidPtr(&api, NULL));
+    add_orbit_constants(mod);
 
     corbamod = Py_InitModule("ORBit.CORBA", corba_functions);
     Py_INCREF(corbamod);
@@ -446,6 +479,8 @@ initORBit(void)
 
     PyModule_AddObject(corbamod, "TRUE",  Py_True);
     PyModule_AddObject(corbamod, "FALSE", Py_False);
+    PyModule_AddObject(corbamod, "Policy",
+		       (PyObject *)&PyCORBA_Policy_Type);
 
     pyorbit_register_exceptions(corbamod);
 
@@ -456,10 +491,17 @@ initORBit(void)
     Py_INCREF(psmod);
     PyModule_AddObject(mod, "PortableServer", psmod);
     PyDict_SetItemString(modules_dict, "PortableServer", psmod);
+    PyModule_AddObject(psmod, "ObjectAdaptor",
+		       (PyObject *)&PyORBit_ObjectAdaptor_Type);
     PyModule_AddObject(psmod, "POA",
 		       (PyObject *)&PyPortableServer_POA_Type);
     PyModule_AddObject(psmod, "POAManager",
 		       (PyObject *)&PyPortableServer_POAManager_Type);
     PyModule_AddObject(psmod, "Servant",
 		       (PyObject *)&PyPortableServer_Servant_Type);
+
+    PyModule_AddIntConstant(psmod, "ORB_CTRL_MODEL", PortableServer_ORB_CTRL_MODEL);
+    PyModule_AddIntConstant(psmod, "SINGLE_THREAD_MODEL", PortableServer_SINGLE_THREAD_MODEL);
+    PyModule_AddIntConstant(psmod, "MAIN_THREAD_MODEL", PortableServer_MAIN_THREAD_MODEL);
+
 }
