@@ -21,7 +21,16 @@
 #  include <config.h>
 #endif
 
-#define ORBIT2_INTERNAL_API
+/* macro to simplify conditionals based on ORBit version */
+#include <orbit/orbit-config.h>
+#define ORBIT_VERSION_CHECK(x,y,z) \
+  (ORBIT_MAJOR_VERSION > x || \
+   (ORBIT_MAJOR_VERSION == x && (ORBIT_MINOR_VERSION > y || \
+     (ORBIT_MINOR_VERSION == y && ORBIT_MICRO_VERSION >= z))))
+
+#if !ORBIT_VERSION_CHECK(2,7,0)
+#  define ORBIT2_INTERNAL_API
+#endif
 #include "pyorbit-private.h"
 #include <structmember.h>
 #include <orbit/poa/poa-types.h>
@@ -78,7 +87,7 @@ _pyorbit_register_skel(ORBit_IInterface *iinterface)
 
     info = g_new0(PyORBitInterfaceInfo, 1);
     info->iinterface = iinterface;
-#if  (ORBIT_MAJOR_VERSION > 2 || (ORBIT_MAJOR_VERSION == 2 && ORBIT_MINOR_VERSION >= 7))
+#if ORBIT_VERSION_CHECK(2,7,0)
     info->class_info.impl_finder = &impl_finder_func;
 #else
     info->class_info.small_relay_call = &impl_finder_func;
@@ -170,6 +179,9 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
     PyObject *args = NULL, *ret = NULL;
     gint n_args, n_rets, i, argpos, retpos;
     gboolean has_ret;
+    PyGILState_STATE state;
+
+    state = pyorbit_gil_state_ensure();
 
     pyservant = SERVANT_TO_PYSERVANT(servant);
     imethod = (ORBit_IMethod *)impl;
@@ -344,6 +356,7 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
     Py_XDECREF(method);
     Py_XDECREF(args);
     Py_XDECREF(ret);
+    pyorbit_gil_state_release(state);
 }
 
 static ORBitSmallSkeleton
@@ -416,6 +429,7 @@ pyorbit_servant_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 
     servant = PYSERVANT_TO_SERVANT(self);
     servant->vepv = info->vepv;
+    ORBit_classinfo_register(&info->class_info);
     ORBIT_SERVANT_SET_CLASSINFO(servant, &info->class_info);
 
     CORBA_exception_init(&ev);
